;---- iNESヘッダ
    .inesprg 1 ; PRG 16KB
    .ineschr 1 ; CHR 8KB
    .inesmap 0 ; マッパー0
    .inesmir 0 ; 水平ミラーリング


;---- 変数の宣言
    .rsset $0000
buttons .rs 1  ; コントローラーの入力を保存する
subPalNo .rs 1 ; サブパレットの番号
posX .rs 1     ; スプライトのX座標
posY .rs 1     ; スプライトのY座標


;---- bank0
    .bank 0
    .org $C000 
RESET:
    sei
    cld

    ; グラフィックを使うには起動後にVBlankを2回待つ必要がある
    ; お約束コードです
.vblankWait1   ; VBlank待ちその1
    bit $2002
    bpl .vblankWait1

.vblankWait2   ; VBlank待ちその2
    bit $2002
    bpl .vblankWait2

    ; これ以降グラフィック関連の処理ができる


    ; 変数の初期化
    lda #0
    sta subPalNo

    lda #100
    sta posX
    sta posY


    ; 2つのパレットを設定

    ; 最初にBGパレットを設定

    ; PPUステータスをリセット (お約束)
    lda $2002
    ; まずアドレス$3F00の上位バイト$3Fを書き込む
    lda #$3F
    sta $2006
    ; 次にアドレス$3F00の下位バイト$10を書き込む
    lda #$00
    sta $2006

    ; Xレジスタを0で初期化。ループ処理のカウンタとして使う。
    ldx #0
.loadBgPalette
    ; bgPalette + x のアドレスの値をロードして、$2007にストア
    lda bgPalette, x
    sta $2007

    ; 16バイト分ループする
    inx
    cpx #16
    bne .loadBgPalette


    ; 続いてスプライト用パレットを設定

    ; PPUステータスをリセット (お約束)
    lda $2002
    ; まずアドレス$3F10の上位バイト$3Fを書き込む
    lda #$3F
    sta $2006
    ; 次にアドレス$3F10の下位バイト$10を書き込む
    lda #$10
    sta $2006

    ; Xレジスタを再度0で初期化。
    ldx #0
.loadSpritePalette
    ; spritePalette + x のアドレスの値をロードして、$2007にストア
    lda spritePalette, x
    sta $2007

    inx
    cpx #16
    bne .loadSpritePalette


    ; スプライトを1つ表示する

    ; Y座標
    lda posY
    sta $0200 

    ; タイル$20番を表示する。
    lda #$20
    sta $0201

    ; 属性の指定。サブパレットは%00。画像の反転などはしない。
    lda #$00
    sta $0202

    ; X座標
    lda posX
    sta $0203


    ; 以下PPUのコントロールポートの設定
    ; NMIを発生させる。スプライトはPPUアドレス$1000からロード
    lda #%10001000
    sta $2000

    ; スプライトを有効にする。
    lda #%00011000
    sta $2001


    ; 無限ループに入って、あとはNMIハンドラで処理する。
.forever
    jmp .forever
  

NMI:
    ; スプライトDMA転送 ($0200-$02FFからPPUのOAMに転送)
    lda #$00
    sta $2003
    lda #$02
    sta $4014

    ; 1Pのコントローラーの入力を取得する
    jsr readOneCon

    ; Aボタンの入力をチェック
    lda buttons
    and #%10000000
    beq .up

    ; Aボタンが押されていたらスプライトのサブパレットを変更
    jsr changeSubPalette
.up
    ; 以下十字キーが入力されてたらスプライトを移動
    ; 十字キーの入力チェック (上)
    lda buttons
    and #%00001000 ; up
    beq .down
    dec posY
.down
    ; 十字キーの入力チェック (下)
    lda buttons
    and #%00000100 ; down
    beq .left
    inc posY
.left
    ; 十字キーの入力チェック (左)
    lda buttons
    and #%00000010 ; left
    beq .right
    dec posX
.right
    ; 十字キーの入力チェック (右)
    lda buttons
    and #%00000001 ; right
    beq .update
    inc posX

.update
    ; スプライトの座標を更新
    ; Y座標
    lda posY
    sta $0200 

    ; X座標
    lda posX
    sta $0203

.done
    rti

IRQ:
    rti


;---- パレットの定義

;---- スプライト用パレット
;---- スプライト用パレット
spritePalette:
    .byte $0f, $00, $10, $30
    .byte $0f, $01, $21, $31
    .byte $0f, $06, $16, $26
    .byte $0f, $09, $19, $29

;---- BG用パレット
bgPalette:
    .byte $0f, $00, $10, $30
    .byte $0f, $01, $21, $31
    .byte $0f, $06, $16, $26
    .byte $0f, $09, $19, $29


;---- 1Pのコントローラー入力を取得するサブルーチン
readOneCon:
    ; 以下はお約束
    lda #$01
    sta $4016
    lda #$00
    sta $4016

    ; 8回入力を読み込むのでカウンタ(Xレジスタ)を8に初期化
    ldx #$08
.loop
    ; ボタンの状態をロードしてビット0をCarryフラグにセット
    lda $4016
    lsr a

    ; buttons変数を左に1ビットシフトする
    ; buttons変数のビット0にはCarryフラグがセットされる
    rol buttons

    ; 8回処理するまで.loopラベルにジャンプ
    dex
    bne .loop
    rts


;---- スプライトのサブパレットを変更
changeSubPalette:
    lda subPalNo
    cmp #3
    bne .nextSubPal
    lda #0
    sta subPalNo
    jmp .setSubPal

.nextSubPal
    inc subPalNo

.setSubPal
    lda subPalNo
    sta $0202
    rts



;---- bank1
    .bank 1
    .org $FFFA
    .word NMI
    .word RESET
    .word IRQ


;---- bank2 (CHR bank0)
;---- 外部ファイル08.chrをパターンテーブルにマッピング
    .bank 2
    .org $0000
    .incbin "08.chr"

